/*

* FreeRTOS for AVR Small demo / Mega Pack *
  

http://avr.paslog.jp/article/1157172.html
2009/06/27 by audin
  
*/

#include "FreeRTOS.h"
#include "task.h"

typedef unsigned char BYTE;
#define setBit(bit,reg) ( reg |= (BYTE)_BV( bit ) )
#define clrBit(bit,reg) ( reg &=( (BYTE)~_BV( bit ) ) )

/* ATmega-Changes */ 
#if defined(__AVR_ATmega8__)||defined(__AVR_ATmega323__)
#define PORTC0 PC0 
#define PORTC1 PC1 
#endif  

#define ledOn()				setBit(PORTC1,PORTC)
#define ledOff()			clrBit(PORTC1,PORTC)
#define ledOn2()			setBit(PORTC0,PORTC)
#define ledOff2()			clrBit(PORTC0,PORTC)

extern void vSio_Task( void *pvParameters );
extern void sio_init(void);

void vTask1( void *pvParameters );
void vTask2( void *pvParameters );

static void led_init(void);

/*******************

* main function
  
 *******************/
portSHORT main( void )
{
	sio_init();
	led_init();

	xTaskCreate( vTask1, "Task1", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 1 , NULL);
	xTaskCreate( vTask2, "Task2", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 1 , NULL);
	xTaskCreate( vSio_Task, "vSio_Task", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 2 , NULL);  

	vTaskStartScheduler();

	return 0;
}

/*******************

* led_init
  
 *******************/
static char ledFlag = 0;
static char ledFlag2 = 0;
static void led_init(void)
{
  setBit(PORTC1,DDRC);
  setBit(PORTC0,DDRC);
}
/*******************

* task1 LED
  
 *******************/
void vTask1( void *pvParameters )
{
  ( void ) pvParameters;	// not used
  
  while(1){
	ledFlag ^= 0xFF; 
  	if(ledFlag){
		ledOn();
  	}
  	else{
		ledOff();
  	}
  	vTaskDelay(1000 / portTICK_RATE_MS);
  }
}
/*******************

* task2 LED
  
 *******************/
void vTask2( void *pvParameters )
{
  ( void ) pvParameters;	// not used
  while(1){
	ledFlag2 ^= 0xFF; 
  	if(ledFlag2){
		ledOn2();
  	}
  	else{
		ledOff2();
  	}
  	vTaskDelay( 300 / portTICK_RATE_MS);
  }
}

